#!/usr/bin/env python

# http://www.securityfocus.com/archive/1/296122/2002-10-19/2002-10-25/2

import dpkt
import fragroute
             
class SynPlus(fragroute.Fragroute):
    def __init__(self):
        fragroute.Fragroute.__init__(self)
        self.op.add_option('-f', action='callback', callback=self.parse_flags,
                           dest='flags', type='string',
                           default=dpkt.tcp.TH_SYN|dpkt.tcp.TH_FIN,
                           help='TCP open flags (default "SF")')

    def parse_flags(self, option, opt, value, parser):
        tcpflags = { 'S':dpkt.tcp.TH_SYN, 'A':dpkt.tcp.TH_ACK,
                     'F':dpkt.tcp.TH_FIN, 'P':dpkt.tcp.TH_PUSH,
                     'R':dpkt.tcp.TH_RST, 'U':dpkt.tcp.TH_URG,
                     'W':dpkt.tcp.TH_CWR, 'E':dpkt.tcp.TH_ECE }
        flags = 0
        for c in value:
            flags |= tcpflags[c]
        setattr(parser.values, option.dest, flags)
    
    def transform(self, ip):
        if ip.p == dpkt.ip.IP_PROTO_TCP and ip.tcp.flags & dpkt.tcp.TH_SYN:
            ip.tcp.flags = self.opts.flags
            ip.sum = ip.tcp.sum = 0
        return [ ip ]

if __name__ == '__main__':
    fragroute.main()
